package com.flow.entity.node;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.flow.enums.TimeUnitEnum;
import com.google.common.collect.Lists;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.flowable.bpmn.model.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;

@EqualsAndHashCode(callSuper = true)
@Data
public class HttpNode extends Node {
    private String url;
    private String method;
    private Map<String, Object> headers;
    private Map<String, Object> requestBody;
    private Long requestTimeout = 0L;
    private Integer retryCount;
    private TimeUnitEnum unit = TimeUnitEnum.MINUTE;
    private Integer duration;


    @Override
    public List<FlowElement> convert() {
        ArrayList<FlowElement> elements = Lists.newArrayList();
        HttpServiceTask httpServiceTask = new HttpServiceTask();
        httpServiceTask.setId(this.getId());
        httpServiceTask.setType("http");
        httpServiceTask.setName(this.getName());
        httpServiceTask.setExecutionListeners(this.buidEventListener());
        // httpServiceTask.setAsynchronous(true);
        // httpServiceTask.setParallelInSameTransaction(true);
        // 失败重试
        boolean isRetry = Objects.nonNull(this.retryCount) && this.retryCount > 0;
        if (isRetry) {
            StringBuilder retry = new StringBuilder();
            retry.append(String.format("R%s", this.retryCount));
            if (Objects.nonNull(this.duration)) {
                retry.append("/")
                        .append(String.format(this.unit.getUnit(), this.duration));
                ExtensionElement retryElement = new ExtensionElement();
                retryElement.setNamespace("https://flowable.org/model");
                retryElement.setNamespacePrefix("flowable");
                retryElement.setName("failedJobRetryTimeCycle");
                retryElement.setElementText(retry.toString());
                httpServiceTask.addExtensionElement(retryElement);
            }
        }
        // 请求参数
        ArrayList<FieldExtension> fieldExtensions = new ArrayList<>();
        FieldExtension urlExtension = new FieldExtension();
        urlExtension.setFieldName("requestUrl");
        urlExtension.setExpression(this.url);
        fieldExtensions.add(urlExtension);
        FieldExtension methodExtension = new FieldExtension();
        methodExtension.setFieldName("requestMethod");
        methodExtension.setStringValue(this.method);
        fieldExtensions.add(methodExtension);
        if (CollectionUtils.isNotEmpty(this.headers)) {
            FieldExtension headerExtension = new FieldExtension();
            headerExtension.setFieldName("requestHeaders");
            headerExtension.setExpression(this.headers.toString());
            fieldExtensions.add(headerExtension);
        }
        if (CollectionUtils.isNotEmpty(this.requestBody)) {
            FieldExtension bodyExtension = new FieldExtension();
            bodyExtension.setFieldName("requestBody");
            bodyExtension.setExpression(this.requestBody.toString());
            fieldExtensions.add(bodyExtension);
        }
        FieldExtension timeoutExtension = new FieldExtension();
        timeoutExtension.setFieldName("requestTimeout");
        timeoutExtension.setStringValue(this.requestTimeout.toString());
        fieldExtensions.add(timeoutExtension);
        FieldExtension failCodesExtension = new FieldExtension();
        failCodesExtension.setFieldName("failStatusCodes");
        failCodesExtension.setStringValue("400,401,403,404,500,503");
        fieldExtensions.add(failCodesExtension);
        FieldExtension handleCodesExtension = new FieldExtension();
        handleCodesExtension.setFieldName("handleStatusCodes");
        handleCodesExtension.setStringValue("200");
        fieldExtensions.add(handleCodesExtension);
        FieldExtension saveResponseBodyExtension = new FieldExtension();
        saveResponseBodyExtension.setFieldName("saveResponseVariableAsJson");
        saveResponseBodyExtension.setStringValue("true");
        fieldExtensions.add(saveResponseBodyExtension);
        FieldExtension activityBehaviorExtension = new FieldExtension();
        activityBehaviorExtension.setFieldName("httpActivityBehaviorClass");
        activityBehaviorExtension.setStringValue("com.flow.behavior.CustomHttpServiceTaskActivityBehavior");
        fieldExtensions.add(activityBehaviorExtension);
        httpServiceTask.setFieldExtensions(fieldExtensions);
        FieldExtension ignoreExtension = new FieldExtension();
        ignoreExtension.setFieldName("ignoreException");
        ignoreExtension.setStringValue(Boolean.toString(isRetry));
        fieldExtensions.add(ignoreExtension);
        elements.add(httpServiceTask);
        // 下一个节点的连线
        Node child = this.getChild();
        SequenceFlow sequenceFlow = this.buildSequence(child);
        elements.add(sequenceFlow);
        // 下一个节点
        if (Objects.nonNull(child)) {
            child.setBranchId(this.getBranchId());
            List<FlowElement> flowElements = child.convert();
            elements.addAll(flowElements);
        }
        return elements;
    }
}
